#!/usr/bin/env python
import logging
import optparse
import os
import os.path
from sys import exit
import time

from sofiehdfformat.core.SofieCoreLogger import setLogger
from sofiehdfformat.core.SofieCsvAccess import SofieCsvAccess
from sofiehdfformat.core.SofieCsvParser import parse_sample_interpret as csv_sample_interpret
from sofiehdfformat.core.SofiePyTableAccess import SofiePyTableAccess
from sofiehdfformat.core.SofieTableStructure import RunMetaName
from sofiehdfformat.core.SofieFileUtils import importdata

DUMPLINLINES = 10

def exportdata(options):
    """
    Function used to export data.
    """
    print '\n-------------------------------\nExporting information' + \
        '\n-------------------------------\n'
    if(options.runname == None):
        exportpath = '/'
    else:
        exportpath = options.runname
    if(options.infile == None):
        print "Specify the in file (--infile filename)"
        exit()
    if(options.outdir == None):
        print "Specify the out file (--exportdir directoryname)"
        exit()
    extension = os.path.splitext(options.infile)
    extension = extension[1]

    if extension != '.h5':
        print "Can only export .h5 files: " + options.infile
        exit()
    else:
        theReader = SofiePyTableAccess(options.infile, runName='/')
        theRuns = theReader.getRuns(exportpath)
        for run in theRuns:
            exportFile = os.sep.join([options.outdir] + run.split('/')) + '.csv'
            if not os.path.isdir(os.path.dirname(exportFile)):
                os.makedirs(os.path.dirname(exportFile))
            if os.path.isfile(exportFile):
                os.unlink(exportFile)
            print '\n-------------------------------\n' + \
                'Exporting: ' + run + ' to ' + str(exportFile)

            theReader.setTable(run, '')
            fieldNames = theReader.getTableStructure(asList=True)
            csvWriter = SofieCsvAccess(exportFile, fieldNames)
            for row in theReader:
                logging.debug("Read New Row")
                parsed = csv_sample_interpret(row)
                csvWriter.write(parsed)
            #aparsedwritercsv = SofieCsvAccess(csvOutParsed)
    csvWriter.close()
    theReader.close()
    
def exportFile(options):
    """
    Function used to export data.
    """
    print '\n-------------------------------\nExporting information' + \
        '\n-------------------------------\n'
    if(options.runname == None):
        print "Specify the complete node/run to export (--runname runNameOrPath)"
        exportpath = options.runname
    if(options.infile == None):
        print "Specify the in file (--infile filename)"
        exit()
    if(options.outfile == None):
        print "Specify the out file (--outfile theExportedFileName    )"
        exit()
    extension = os.path.splitext(options.infile)
    extension = extension[1]
    if extension != '.h5':
        print "Can only export .h5 files: " + options.infile
        exit()
    else:
        theReader = SofiePyTableAccess(options.infile)
        theReader.readFile(options.runname, options.outfile)
        theReader.close()

def listdata(options):
    if(options.runname == None):
        options.runname = '/'
    if(options.infile == None):
        print "Specify the in file (--infile filename)"
        exit()
    extension = os.path.splitext(options.infile)
    extension = extension[1]
    if extension != '.h5':
        print "Can only lsit .h5 files: " + options.infile
        exit()
    theReader = SofiePyTableAccess(options.infile, runName='/')
    runs = theReader.getRuns(options.runname)
    runs.sort()
    for run in runs:
        logging.debug('Setting Run: ' + run);
        splitRun = os.path.split(run);
        theRun, theNode = splitRun
        theReader.setRunName(theRun)
        meta = theReader.getRunMeta(options.runname)
        if meta:
            run, description, timestamp = meta[0]
        else:
            run, description, timestamp = None, None, None
        print "\n\n==============================="
        print "==============================="
        print "RUN: " + theRun + '/' + theNode + "\n-------------------------------"
        if theNode == RunMetaName:

            print "Description: " + str(description)
            print "Time: " + time.strftime('%D,  %H:%M:%S',
                                           time.localtime(timestamp))
        elif options.verbose:
            theReader.setTable(theNode, '')
            x = DUMPLINLINES
            for row in theReader:
                print str(row).lstrip('(').rstrip(')')
                if x == 0:
                    print  '.... ONLY THE FIRST ' + \
                        str(DUMPLINLINES) + ' ENTRIES ARE PRINTED ....'
                    break;
                x -= 1;
            
    else:
        print "\n\n==============================="
        print "==============================="
        print "RUN: " + options.runname + ' DOES NOT EXIST' + \
            "\n-------------------------------"
    theReader.close()

def deletedata(options):
    if(options.runname == None):
        print "Specify the rest run name (--runname 01CornerTestRun) "
        exit()
    extension = os.path.splitext(options.infile)
    extension = extension[1]
    if extension != '.h5':
        print "Can only lsit .h5 files: " + options.infile
        exit()
    theReader = SofiePyTableAccess(options.infile, runName='/')
    if options.recursive:
        recursively = ' (recursively enabled) '
    else:
        recursively = ' (recursively disabled)'
    if theReader.deleteRun(options.runname, options.recursive):
        print "\n\n==============================="
        print "==============================="
        print "RUN: " + options.runname + ' DELETED' + recursively + \
            "\n-------------------------------"
    else:
        print "\n\n==============================="
        print "==============================="
        print "RUN: " + options.runname + ' NOT DELETED' + recursively + \
            "\n-------------------------------"
    theReader.close()

if __name__ == '__main__':
    usage = """usage: %prog [options] arg1
This package is used to manipulate various measurement file and manage
experiments.

Currently supported import formats are:

1. C3D (.c3d) Motion capture files.
2. CSV (.csv) with all values as numeric.
3. GCE (.gce) from the http://chomsky.shampoo.ca/goldenembed/ ANT logger project.
4. ANT (.ant) from the https://github.com/mvillalba/python-ant ANT USB logger project.
4. ADAMS (.tab) Output format from Msc Software ADAMS using the table export utility.

The different arguments are:
    import -> Imports data into HDF5 Format.
    export -> Exports data from an HDF5 File.
    exportFile -> Exports a file node within a file to an external file.
    list -> Lists information about the file.
    delete -> Delete a node from the file or run from the file.
    """
    parser = optparse.OptionParser(usage=usage)
    parser.add_option('--infile', '-i', default=None,
                      help="The File to import into HDF format or the H5 file that you want" +
                      " to manipulate with export/list/update")
    parser.add_option('--outfile', '-o', default=None,
                      help="The HDF file where the data will be saved. It must have an '.h5'" +
                      "extension")
    parser.add_option('--exportdir', '-e', default=None, help="The directory " + \
                      "where the files will be exported to, (for export)")
    parser.add_option('--runname', '-t', default=None,
                      help="The Run that you are doing.")
    parser.add_option('--description', '-d', default=None,
                      help="The Run Description that you are doing.")
    parser.add_option('--verbose', '-v', dest="verbose",
                      default=0, help="Enable verbose mode. 0, little, 1 some, 2 debug.")
    parser.add_option("-p", "--parse",
                      action="store_true", default=False,
                      help="Parse the data into correct format.")
    parser.add_option("-c", "--csv",
                      action="store_true", default=False,
                      help="Output the data to a CSV File.")
    parser.add_option("-r", "--recursive",
                      action="store_true", default=False,
                      help="Recursively delete information.")
    
    options, arguments = parser.parse_args()
    if int(options.verbose) == 2:
        setLogger(logging.DEBUG)
        logging.debug('Enabled DEBUG MODE')
    if len(arguments) == 0:
        print "Specify the command to follow, or -h for help."
        exit()
    elif arguments[0] == 'import':
        importdata(options.infile, options.outfile, options.runname,
                   options.description, options.parse, options.csv)
        print "\n\n==============================="
        print "==============================="
        print "RUN: " + options.runname + ' IMPORTED' + \
            "\n-------------------------------"
    elif arguments[0] == 'export':
        exportdata(options)
    elif arguments[0] == 'exportFile':
        exportFile(options)
    elif arguments[0] == 'list':
        listdata(options)
    elif arguments[0] == 'delete':
        deletedata(options)

    
        
