import logging
import time

from sofiehdfformat.c3dpytableaccess.C3DTableStructure import C3DMarker
from sofiehdfformat.c3dpytableaccess.C3DTableStructure import C3DMarkerName
from sofiehdfformat.c3dpytableaccess.C3DTableStructure import C3DMeta
from sofiehdfformat.c3dpytableaccess.C3DTableStructure import C3DMetaName
from sofiehdfformat.core.SofiePyTableAccess import SofiePyTableAccess
from tables import *
#import numpy.rec
class C3DDataAccess(SofiePyTableAccess):
    """ Provides the class to write C3D columns to the table in the SOFIE HDF Format file. """
    def __init__(self, filename, runName='/test',
                 allowDuplicates=True,
                 description='',
                 videoSamplingRate=-1,
                 videoFrames=-1,
                 scale=-0.1):
        SofiePyTableAccess.__init__(self, filename, runName=runName,
                                    allowDuplicates=allowDuplicates,
                                    description=description);
        self.setTable(C3DMetaName, C3DMeta)
        SofiePyTableAccess.write(self, [videoSamplingRate, videoFrames, scale,
                                 time.time()])
        self.allowDuplicates()
    
    def write(self, theMarkers):
        """ Writes the data into a table with each Marker Label in a separate Table
        """
        #logging.debug(theMarkers)
        if theMarkers == False:
            return
        for key in theMarkers.keys():
            theMarker = theMarkers[key]
            key = key.replace('*', '_STAR')

            #logging.debug('The Marker'+str(theMarker))
            #logging.debug('Writing Marker: '+key)

            self.setTable(C3DMarkerName + key, C3DMarker)
            SofiePyTableAccess.write(self, theMarker)

    def __iter__(self):
        self.tablesForIteration = [table for table in self.hdfFile.listNodes(self.run, 'Table') if
            table.name.find(C3DMarkerName) == 0]
        self.iterIndex = -1
        return self

    def next(self):
        theValues = {}
        self.iterIndex += 1
        for table in self.tablesForIteration:
            if self.iterIndex == table.nrows:
                raise StopIteration
            row = table[self.iterIndex]
            logging.debug('Writing Rows: '+str(self.iterIndex)+':'+str(row))
            for column in table.colnames:
                theValues[table.name[len(C3DMarkerName):]+'_'+column] = row[column]
        #logging.debug('Read Row: '+str(theValues))
        return theValues
