"""
Implements a wrapper to the CSV File format.

This allows you to write from all the readers that are created
to a CSV format to allow other people to use it if they
do not want to use HDF5.

Vitables can also export individual files to CSV.
"""
import logging
import csv
csv.QUOTE_NONNUMERIC
class SofieCsvAccess(object):
    
    def __init__(self,filename,fieldNames=False):
        logging.debug('Opening File: {0}'.format(filename))
        self.csvFile = open(filename, mode = "wb")
        if fieldNames:
            self.writer = csv.DictWriter(self.csvFile,fieldNames)
            self.writer.writeheader()
            self.writer = csv.writer(self.csvFile)
            self.headerWritten = True
        else:
            self.writer = csv.writer(self.csvFile)
            self.headerWritten = False

    def __del__(self):
        self.csvFile.close()

    def __iter__(self):
        return self.writer
    
    def write(self,theDataList):
        """
        """
        if theDataList == False:
            return
        if isinstance(theDataList, dict):
            if not self.headerWritten:
                self.writer = csv.DictWriter(self.csvFile,theDataList.keys())
                self.writer.writeheader()
                self.headerWritten = True
        self.writer.writerow(theDataList)

        
    def writelines(self,theListOfData):
        for theDataList in theListOfData:
            self.write(theDataList)

    def close(self):
        self.csvFile.close()        