""" Provides some standard funtions for importing,exporting data from sofiehdfformat files. """
import logging
import os
from sys import exit
from sofiehdfformat.antparser.AntParser import parse_sample as ant_sample
from sofiehdfformat.antparser.AntParser import parse_sample_interpret as ant_sample_interpret
from sofiehdfformat.antpytableaccess.AntPyTableAccess import AntParsedDataAccess
from sofiehdfformat.antpytableaccess.AntPyTableAccess import AntRawDataAccess
from sofiehdfformat.c3dparser.C3dParser import parse_sample as c3d_sample
from sofiehdfformat.c3dparser.C3dParser import parse_sample_interpret as c3d_sample_interpret
from sofiehdfformat.c3dpytableaccess.C3DPyTableAccess import C3DDataAccess
import sofiehdfformat.c3dreader.c3dreader as C3
import sofiehdfformat.core.SofieAdamFile as adams
from sofiehdfformat.core.SofieCsvAccess import SofieCsvAccess
import sofiehdfformat.core.SofieCsvFile as scf
from sofiehdfformat.core.SofieCsvParser import parse_sample as csv_sample
from sofiehdfformat.core.SofieCsvParser import parse_sample_interpret as csv_sample_interpret
from sofiehdfformat.core.SofieCsvPyTableAccess import SofieCsvPyTableAccess
from sofiehdfformat.core.SofiePyTableAccess import SofiePyTableAccess
from sofiehdfformat.core.SofieHdfFormatExceptions import AntRawParserLengthNotCorrect
from sofiehdfformat.core.SofieHdfFormatExceptions import FileNotSupportedException
from sofiehdfformat.core.config import getGESubDirectory, getSparkSubDirectory,\
    getC3DSubDirectory, getCSVImportSubDirectory, getAdamsSubDirectory,\
    getBagSubDirectory,getDefaultBagFile,getDefaultBagFileName
import sofiehdfformat.gefilereader.GeFile as GE
import sofiehdfformat.sparkfunantusbreader.SparkFunFile as SF
import tempfile
import sys
TMPCSVFILENAME='ar-csv.csv'
#THE DEFAULT TABLE STRUCTURE FOR RECORDING QUATERNION DATA
defaultQuaternionTableStructure = \
    ['id', 'confidence', 'x', 'y', 'z', 'quat1', 'quat2', 'quat3', 'quat4', 'timestamp']
defaultAngleTableStructure = \
    ['angle', 'timestamp']


def importdata(infile, outfile, runName, description, shouldParse, shouldCsv):
    """
    Standard function used to import data into the file
    """
    print '\n-------------------------------\nImporting information' + \
        '\n-------------------------------\n'
    if(infile == None):
        print "Specify the in file (--infile filename)"
        exit()
    if(outfile == None):
        print "Specify the out file (--outfile filename)"
        exit()
    if(runName == None):
        print "Specify the rest run name (--runName 01CornerTestRun) "
        exit()

    extension = os.path.splitext(infile)
    infileName = extension[0]
    extension = extension[1]

    csvOut = infileName + '-sofie.csv'
    csvOutParsed = infileName + '-sofie-parsed.csv'

    if extension == '.gce':
        runNameExtended = getGESubDirectory(runName)
        theOriginalFile = GE.open(infile)
        parser = ant_sample
        parser_interpret = ant_sample_interpret
        theWriter = AntRawDataAccess(outfile,
                                     runName=runNameExtended,
                                     description=description)
    elif extension == '.ant':
        runNameExtended = getSparkSubDirectory(runName)
        theOriginalFile = SF.open(infile)
        parser = ant_sample
        parser_interpret = ant_sample_interpret
        theWriter = AntRawDataAccess(outfile,
                                     runName=runNameExtended,
                                     description=description)
    elif extension == '.c3d' or extension == '.C3D':
        runNameExtended = getC3DSubDirectory(runName)
        theOriginalFile = C3.open(infile)
        parser = c3d_sample
        parser_interpret = c3d_sample_interpret
        theWriter = C3DDataAccess(outfile,
                                  runName=runNameExtended,
                                  description=description,
                                  allowDuplicates=True,
                                  videoSamplingRate=theOriginalFile.getVideoSamplingRate(),
                                  videoFrames=theOriginalFile.getVideoFrames(),
                                  scale=theOriginalFile.getScale())
    elif extension == '.csv':
        runNameExtended = getCSVImportSubDirectory(runName)
        theOriginalFile = scf.open(infile)
        parser = csv_sample
        parser_interpret = csv_sample_interpret
        theWriter = SofieCsvPyTableAccess(outfile,
                                          runName=runNameExtended,
                                          description=description,
                                          fieldNames=theOriginalFile.getTableStructure())
    elif extension == '.tab':
        runNameExtended = getAdamsSubDirectory(runName)
        theOriginalFile = adams.open(infile)
        parser = csv_sample
        parser_interpret = csv_sample_interpret
        theWriter = SofieCsvPyTableAccess(outfile,
                                          runName=runNameExtended,
                                          description=description,
                                          fieldNames=theOriginalFile.getTableStructure())
    elif extension == '.bag':
        runNameExtended = getBagSubDirectory(runName)
        theWriter = SofieCsvPyTableAccess(outfile,
                                          runName=runNameExtended,
                                          description=description,
                                          fieldNames=['A HACK'])
        theWriter.writeFile(infile)
        print '\n-------------------------------\n{0} Saved  to file'.format(outfile)
        theWriter.close()
        return
    else:
        raise FileNotSupportedException('File not Supported: ' + str(extension))
    print '\n-------------------------------\nSaving to file',
    print outfile
    if shouldParse:
        print 'File is going to be parsed'
        print 'CSV Parsed File: ' + csvOutParsed
    if shouldCsv:
        print 'CSV File: ' + csvOut + '\n-----------------------------------\n'
        awritercsv = SofieCsvAccess(csvOut)

    for theLine in theOriginalFile:
        logging.debug(theLine)
        try:
            parsedLine = parser(theLine)

        except Exception, e:
            print 'Parsing Failed: ' + str(e)
            continue
        try:
            logging.debug(parsedLine)
            theWriter.write(parsedLine)
            print '.',
            if shouldCsv:
                awritercsv.write(parsedLine)
                print 'c',
        except AntRawParserLengthNotCorrect, e:
            print 'x'


    if shouldParse:
        if extension == '.gce':
            parsedWriter = AntParsedDataAccess(outfile,
                                               runNameExtended)
        elif extension == '.ant':
            parsedWriter = AntParsedDataAccess(outfile,
                                               runNameExtended)
        elif extension == '.c3d':
            parsedWriter = None
        elif extension == '.csv':
            parsedWriter = None
        else:
            raise FileNotSupportedException('File not Supported: ' + extension)

        print '\n-------------------------------\nParsing Data\n'

        if shouldCsv:
            aparsedwritercsv = SofieCsvAccess(csvOutParsed)
        previousParsedLine = None
        for row in theWriter:
            parsedLine = parser_interpret(row, previousParsedLine)
            logging.debug('The parsedLine: ' + str(row))
            if shouldCsv:
                aparsedwritercsv.write(parsedLine)
                print 'c',
            if parsedWriter:
                parsedWriter.write(parsedLine)
                print 'p',
            previousParsedLine = parsedLine

    print 'DUPLICATE WRITES: ' + str(theWriter.getDuplicateWrites())
    theWriter.close()
    
def exportFile(filename,runName,outFilename):
    """ Exports a file node from a sofiehdfformat file. """
    theReader = SofiePyTableAccess(filename)
    theReader.readFile(runName, outFilename)
    theReader.close()
    
def createTempAndFileDirectory(filename):
    ''' Create a temp directory with filename as the name of the file appended
    to the filename.
    '''
    tmpdir = tempfile.mkdtemp()
    return os.path.join(tmpdir, filename)
def cleanUpTempDirectory(tempDirectoryWithFilename):
    ''' Clean Up temp directory with filename as the extra file '''
    if os.path.isfile(tempDirectoryWithFilename):
        os.remove(tempDirectoryWithFilename)
    if os.path.isdir(os.path.abspath(tempDirectoryWithFilename)):
        os.rmdir(os.path.abspath(tempDirectoryWithFilename))
        

def importARData(csvFileName,hdfFilename,runName):
    """ Used to import AR csv file """
    print 'IMPORTING AR DATA'
    try:
        if os.path.isfile(csvFileName) and os.path.getsize(csvFileName) > 0:
            importdata(csvFileName,
                hdfFilename,
                runName,
                'description',
                True,
                False)
        print 'FINISHED PARSING AR INTO SOFIEHDFFILE.'
    except:
        print "Unexpected error:", sys.exc_info()[0]
        raise
    return True

def importBagData(usbCamBagFilename,hdfFilename,runName):
    """ Imports a bag file (from ROS) into the sofiehdfformat file, as a file node. """
    if os.path.isfile(usbCamBagFilename) and os.path.getsize(usbCamBagFilename) > 0:
        importdata(usbCamBagFilename,
                    hdfFilename,
                    runName,
                    'description',
                    True,
                    False)
def exportBagData(filename,runName):
    """ Exports the bag file from the sofiehdfformat file. """
    logging.debug(runName)
    totalRun =  getDefaultBagFile(getBagSubDirectory(runName))
    logging.debug(totalRun)
    outFileName = filename+'_'+getDefaultBagFileName()
    exportFile(filename,totalRun,outFileName)
    return outFileName