import logging
import os
import re
import time

from sofiehdfformat.core.SofieHdfFormatExceptions import SetTheTableFirstException,\
    MetaTableCorruptedException
from sofiehdfformat.core.SofieTableStructure import RunMeta
from sofiehdfformat.core.SofieTableStructure import RunMetaName
from sofiehdfformat.core.SofieTableStructure import Sync
from sofiehdfformat.core.SofieTableStructure import SyncName
from tables import *
from tables.exceptions import NoSuchNodeError
from tables.exceptions import NodeError
from tables.file import openFile
from tables.group import Group
from tables.leaf import Leaf
from tables.table import Table
from tables.nodes import filenode

class SofiePyTableAccess(object):
    """
Implements a wrapper to the PyTables h5 format.

You set a run to write to that with a specific table format.
The iterator of the SofiePyTableAccess object is the row iterator
of the current table.
"""
    def __init__(self,filename,runName='/test',allowDuplicates=False,
        description=''
    ):
        self.table = None
        """ Of Type Node/Group."""
        self.run = None
        extensionOut = os.path.splitext(filename)
        filename = extensionOut[0] + '.h5'
        self.theTableName = None
        self.theTableStructure = None
        self.hdfFile = openFile(filename, mode = "a")
        self.filename = filename
        self.__allowDuplicates = allowDuplicates
        logging.debug('Setting the Run: '+str(self.getRuns()))
        self.setRunName(runName)
        logging.debug('Setting the RunMeta'+str(self.getRuns()))
        self.setRunMeta(runName, description)
        logging.debug('Finished Setting the RunMeta'+str(self.getRuns()))


    def __del__(self):
        self.hdfFile.close()

    def __iter__(self):
        return self.table.__iter__()
    def is_number(self,s):
        try:
            float(s)
            return True
        except ValueError:
            return False

    def next(self):
        return self.table.next()
    
    def allowDuplicates(self):
        self.__allowDuplicates=True

    def doNotAllowDuplicates(self):
        self.__allowDuplicates=False

    def getDuplicateWrites(self):
        return self.duplicates

    @staticmethod
    def _createNiceRunName(runName,fullPath=True):
        if fullPath:
            return '/'+runName.strip('"').strip("'").lstrip('/')
        else:
            return runName.strip('"').strip("'").lstrip('/')
    
    
    def setRunName(self,runName,createNew=True):
        """
        Sets the base path for the leaf to be written/read too. If the
        group does not exist it will be created by default.

        If createNew is False the path will not be created and the run
        will be set to the root directory.
        """
#for node in self.hdfFile:
#    logging.debug(node)
        subRunName = self._createNiceRunName(runName).split('/')
        theRoot = '/'
        for runName in subRunName:
            theRoot_tmp = SofiePyTableAccess._createNiceRunName(theRoot+'/'+runName)
            logging.debug('TMP_RUN:'+theRoot_tmp);

            if theRoot_tmp in self.hdfFile:
                logging.debug('RUN EXISTS: '+theRoot_tmp);
                if isinstance(self.hdfFile.getNode(theRoot_tmp),Leaf):
                    logging.debug('IS LEAF');
                    self.run = self.hdfFile.getNode(
                        os.path.split(theRoot_tmp)[0])
                else:
                    runName = SofiePyTableAccess._createNiceRunName(runName,fullPath=False)
                    theRoot = SofiePyTableAccess._createNiceRunName(theRoot)
                    logging.debug('IS GROUP:'+theRoot+':'+runName);
                    if runName != '/':
                        self.run = self.hdfFile.getNode(theRoot, runName)
                    else:
                        self.run = self.hdfFile.getNode(theRoot)
            elif createNew and theRoot_tmp != '/':
                logging.debug('NEW RUN:'+theRoot+':'+runName);
                self.run = self.hdfFile.createGroup(SofiePyTableAccess._createNiceRunName(theRoot),
                                                    SofiePyTableAccess._createNiceRunName(runName,fullPath=False));
            else:
                logging.warning('Not Creating a new node.')
                self.run = self.hdfFile.getNode(SofiePyTableAccess._createNiceRunName(theRoot))
            theRoot += '/'+runName
        self.duplicates = 0;
        #------------- logging.debug('SETTABLE: '+str(self.getTableStructure()))
        self.setTable(self.getTableName(),self.getTableStructure())

    def getRunName(self):
        return self.run._v_pathname;

    def getRun(self):
        return self.run;
    
    def getRuns(self,baseRun='/'):
        return SofiePyTableAccess.getRunsInTheFile(self.hdfFile,baseRun)
    
    @staticmethod
    def setRunMetaInFile(filenameOrFile,runName,runMeta):
        closeTheFile = False
        if isinstance(filenameOrFile,File):
            hdfFile = filenameOrFile
        else:
            closeTheFile = True;
            try:
                hdfFile = SofiePyTableAccess(filenameOrFile)
            except:
                return []
        runMetaDefault = {'runName':runName,
                    'runExperimentType': '',
                           'runSubject':'',
                           'runObject':'',
                           'runSuccessful':True,
                           'runCorrupted':False,
                           'runNotes':''
                           }
        runMeta = dict(runMetaDefault.items()+runMeta.items())
        hdfFile.setRunMeta(runName=runMeta['runName'],
            runNotes =runMeta['runNotes'],
            runSubject =runMeta['runSubject'],
            runObject =runMeta['runObject'],
            runSuccessful =runMeta['runSuccessful'],
            runCorrupted =runMeta['runCorrupted'],
            runExperimentType =runMeta['runExperimentType'])
        if closeTheFile:
            hdfFile.close();
            
    @staticmethod
    def getRunMetaInFile(filenameOrFile,runName):
        closeTheFile = False
        if isinstance(filenameOrFile,File):
            hdfFile = filenameOrFile
        else:
            closeTheFile = True;
            try:
                hdfFile = SofiePyTableAccess(filenameOrFile)
            except:
                return {}
        try:
            returnRunMeta = hdfFile.getRunMeta(runName)
        except MetaTableCorruptedException:
            return []
        if closeTheFile:
            hdfFile.close()
        return returnRunMeta
    
    @staticmethod
    def getRunsInTheFile(filenameOrFile,baseRun='/'):
        """
        Gets all the Runs in the file (ie, all the tables of all runs.)

        """
        closeTheFile = False
        if isinstance(filenameOrFile,File):
            hdfFile = filenameOrFile
        else:
            closeTheFile = True;
            try:
                hdfFile = openFile(filenameOrFile, mode = "a")
            except:
                return []
        baseRun = SofiePyTableAccess._createNiceRunName(baseRun)
        try:
            theGroup = hdfFile.getNode(baseRun)
            if not isinstance(theGroup,Group):
                return [theGroup._v_pathname]
            runs = [node._v_pathname for node in hdfFile.walkNodes(baseRun)
            if isinstance(node, Table)
            ]
            logging.debug('The Runs: '+str(runs))
            if closeTheFile:
                hdfFile.close();
            return runs
        except NoSuchNodeError:
            logging.warning('Path does not exist: '+baseRun)
            if closeTheFile:
                hdfFile.close();
            return []

    def deleteRun(self,nodeName,recursive=False):
        """
        Deletes the run or Node.
        """
        nodeName = SofiePyTableAccess._createNiceRunName(nodeName).rstrip('/')
        logging.debug('Deleting the followning: '+nodeName+' :'+str(self.getRuns()))
        if nodeName == '/':
            logging.warning('CANNOT DELETE ROOT NODE:'+\
                    nodeName)
            return False;
        try:
            if nodeName in self.getRuns():
                self.hdfFile.removeNode(nodeName,recursive=recursive)
                return True
            else:
                return False
            
        except NoSuchNodeError:
            logging.warning('COULD NOT DELETE (recursive not enabled): '+\
                    nodeName+' NODE DOES NOT EXIST')
            return False
        except NodeError:
            logging.warning('COULD NOT DELETE (recursive not enabled): '+\
                    nodeName)
            return False

    def _runMetaSetup(self):
        currentTableName = self.getTableName()
        currentTableStructure = self.getTableStructure()
        currentRun =  self.getRunName()
        self.setRunName('/')
        logging.debug('The RUN Name: '+self.getRunName())
        self.setTable(RunMetaName,RunMeta)
        return currentRun,currentTableName,currentTableStructure
    
    def _runMetaTearDown(self,currentRun,currentTableName,currentTableStructure):
        logging.debug('Setting the RunName:'+str(self.getRuns()))
        self.setTable(currentTableName,currentTableStructure)
        self.setRunName(currentRun)
        logging.debug('Setting the RunName:'+str(self.getRuns()))
    
    def _getMetaInformationRow(self,fieldName,searchTerm):
        theRowIterator =  self.table.where('{0}=="{1}"'.format(fieldName,
                                                               searchTerm.strip('/')))
        logging.debug(theRowIterator)
        return theRowIterator
    
    def _makeRunNameSafeForMeta(self,runName):
        return runName.strip('/')



    def getRunMeta(self,runName):
        currentRun,currentTableName,currentTableStructure =  self._runMetaSetup()
        runName = self._makeRunNameSafeForMeta(runName)
        if currentRun:
            data = self._getMetaInformationRow('runName',runName)
            dataReturn = [{'runName':row['runName'],
                           'runExperimentType':row['runExperimentType'],
                           'runSubject':row['runSubject'],
                           'runObject':row['runObject'],
                           'runSuccessful':row['runSuccessful'],
                           'runCorrupted':row['runCorrupted'],
                           'runNotes':row['runNotes'],
                           'Timestamp':row['Timestamp'],} for row in data]
            if len(dataReturn)==0:
                return False
            elif len(dataReturn)!=1:
                logging.debug(dataReturn)
                raise MetaTableCorruptedException('Looking for Run: {0}:{1}'.format(
                                                                runName,
                                                                str(dataReturn)))
            dataReturn = dataReturn[0]
        else:
            dataReturn = False
        
        self._runMetaTearDown(currentRun, currentTableName, currentTableStructure)
        return dataReturn

    def setRunMeta(self,runName,
                   runNotes=None,
                   runSubject=None,
                   runObject=None,
                   runSuccessful=None,
                   runCorrupted=None,
                   TimeStamp=None,
                   runExperimentType=None
                   ):
        if not runName:
            logging.debug('RUNNAME is None.')
            return False
        runName = self._makeRunNameSafeForMeta(runName)
        currentRun,currentTableName,currentTableStructure =  self._runMetaSetup()
        logging.debug('currentRun: {0},currentTableName: {1},currentTableStructure: {2}'.
                      format(currentRun,currentTableName,currentTableStructure))
        metaList ={'runName':runName.strip('/'),
                           'runExperimentType':runExperimentType,
                           'runSubject':runSubject,
                           'runObject':runObject,
                           'runSuccessful':runSuccessful,
                           'runCorrupted':runCorrupted,
                           'runNotes':runNotes,
                           'Timestamp':time.time()}
        logging.debug('Setting the Run Meta'+str(metaList)+ ' '+self.getRunName());
        currentRowIterator = self._getMetaInformationRow('runName', runName)
        updating=False
        for currentRow in currentRowIterator:
            logging.debug('Updating RUN META')
            for key in metaList.keys():
                logging.debug('Updating column: {0}:{1}'.format(key,metaList[key]))
                currentRow[key] = metaList[key]
            currentRow.update()
            updating=True
        else:
            self.table.flush()
        if not updating:
            logging.debug('New RUN META')
            allowDuplicates = self.__allowDuplicates
            self.doNotAllowDuplicates()
            self.__write(metaList)
            if allowDuplicates:
                self.allowDuplicates()
        self._runMetaTearDown(currentRun, currentTableName, currentTableStructure)
        return True

    def _makesafekey(self,key):
        return re.sub(r'\W+|_', '', key)
    def _makeSafeValue(self,value):
        if isinstance(value,basestring) and not self.is_number(value):
            return '"'+re.escape(value)+'"'
        return float(value)
    
    def __write(self,theDataList):
        #if not isinstance(theDataList,set):
        #    raise SofiePyTableAccesCanOnlyWriteLists()
        
        row = self.table.row
        columns = self.table.colnames
# logging.debug('The Keys:'+str(columns)+' The Table:'+str(self.table)+\
#     ' The Before Right Row:'+str(row))

        if not self.__allowDuplicates:
            #if isinstance(theDataList, dict):
            #    theDataList = theDataList.values()
            logging.debug(str(theDataList))
            whereString ='('
            for key,value in zip(columns,theDataList):
                logging.debug(key)
                if isinstance(theDataList, dict):
                    if theDataList[key] != None:
                        value = theDataList[key]
                        value = self._makeSafeValue(value)
                        whereString += '('+self._makesafekey(key)+'=='+\
                            str(value)+') & '
                elif value != None:
                    value = self._makeSafeValue(value)
                    whereString += '('+self._makesafekey(key)+'=='+str(value)+') & '
            whereString = whereString[0:-3]+')'
            logging.debug('The whereString: '+whereString)
            for foundRow in self.table.where(whereString):
                logging.debug('DUPLICATEROW:'+str(foundRow))
                self.duplicates += 1
                return False
        if isinstance(theDataList, dict):
            logging.debug('The Keys:'+str(theDataList.keys())+\
                ': The Data:'+str(theDataList))
            for key in theDataList.keys():
                logging.debug('Setting: '+key)
                row[self._makesafekey(key)] = theDataList[key]
        else:
            logging.debug('The Keys:'+str(columns)+\
                ': The Data:'+str(theDataList))
            for key,value in zip(columns,theDataList):
                #logging.debug('Setting: '+key+' : '+str(type(value))+\
                #' : '+ str(value))
                row[self._makesafekey(key)] = value
        row.append();
        self.table.flush()
        return True

    def write(self,theDataList):
        """
        This writes a single entry to the table and does not write the value
        if there is already a value in the table with the same column values.
        """
        self.__write(theDataList)
        
    def writelines(self,theListOfData):
        for theDataList in theListOfData:
            self.write(theDataList)

    def close(self):
        self.hdfFile.flush()
        self.hdfFile.close()

    def getTable(self):
        #------------------------------- logging.debug('SOFIE GET TABLE CALLED')
        return self.table

    def getTableName(self):
        #------------------------------- logging.debug('SOFIE GET TABLE CALLED')
        return self.theTableName

    def getTableStructure(self,asList=False):
        #------------------------ logging.debug('SOFIE GET TABLE COLUMN CALLED')
        if asList:
            return self.table.colnames
        return self.theTableStructure

    def setTable(self,theTableName,theTableStructure):
        self.theTableStructure = theTableStructure
        self.theTableName = theTableName
        
        if theTableName == None:
            return
        elif theTableName == '/':
            self.run = '/'
            return
        elif theTableStructure == None:
            return
        
        logging.debug('The Table: '+str(self.theTableName)+" : "+\
            str(self.theTableStructure)+' : THERUN: '+str(self.run))

        if theTableName in self.run:
            self.table = self.hdfFile.getNode(self.run,theTableName)
            self.theTableStructure = self.table.description
            #logging.debug('The Table:'+str(self.table.description))
            #logging.debug('The columns:'+str(self.theTableStructure))
        else:
            self.table = self.hdfFile.createTable(self.run,
                theTableName,theTableStructure)

    def readlines(self):
        logging.debug(self.run)
        logging.debug(self.table)
        if self.table == None:
            raise SetTheTableFirstException()
        return self.table[:]

    def writeSync(self,timeStamp=None,sample=1):
        """ Writes the data into a table with each byte in a separate column.
        """
        logging.debug(sample)
        if timeStamp==None:
            timeStamp = time.time()
        thePayloadList = {'Timestamp':timeStamp,'Sample':sample}
        currentTableName = self.getTableName()
        tableStructure = self.getTableStructure()
        self.setTable(SyncName,Sync)
        logging.debug('Writing Sync'+str(thePayloadList));
        self.__write(thePayloadList)
        self.setTable(currentTableName,tableStructure)
        return timeStamp
    
    def writeFile(self,fileToImport):
        """
        Used to import a binary file into the HDFFile.
        """
        fnode = filenode.newNode(self.hdfFile, where=self.run, 
                                 name=self._makesafekey(os.path.basename(fileToImport)))
        with open(fileToImport, 'r') as fdisk:
            contents = fdisk.read()
        fnode.write(contents)
        fnode.close()
        
    def readFile(self,fileNodeToRead,outputfileName):
        """
        Used to export a binary file within the HDFFIle to a output file.
        """
        logging.debug(fileNodeToRead)
        fileNodeToRead = os.path.join(os.path.dirname(fileNodeToRead),
                                      self._makesafekey(os.path.basename(fileNodeToRead)))
        logging.debug(fileNodeToRead)
        fileNodeOb = self.hdfFile.getNode(fileNodeToRead)
        fnode = filenode.openNode(fileNodeOb,'r')
        with open(outputfileName, 'w') as fdisk:
            fdisk.write(fnode.read());
            fdisk.close();
        fnode.close()
        

    def getSyncs(self):
        currentTableName = self.getTableName()
        tableStructure = self.getTableStructure()
        self.setTable(SyncName,Sync)
        data = self.readlines()
        self.setTable(currentTableName,tableStructure)
        return data