import logging
"""
Read packets from the GCE File format. used on the sparkfun logger.

"""
import __builtin__
import os
import struct

from ant.core import exceptions as antException
import ant.core.constants as constants
import ant.core.message as message
from sofiehdfformat.core.SofieUtils import convertRawDataToHexString

TIMESTAMP_LEN=3;

def open(filename,mode='rb'):
    return GeFile(filename,mode);    

class GeFile(object):
    def __init__(self, filename,mode):
        self.reader = __builtin__.open(filename,mode)
        
        self.fileSize = os.path.getsize(filename)
        logging.debug('The File'+str(self.reader)+' Its Size: '+
            str(self.fileSize))
        self.totalIndex = 0
        self.message = message.Message();
        
    def readline(self,size=0):
        """
        Reads one sample from the file.

        Returns a dictionary with {'timestamp':unixTime,'packet':packet
        """
        dataIndex = -1
        while (dataIndex != 0) and (self.totalIndex < self.fileSize-1):
            self.reader.seek(self.totalIndex)
            self.rawdata = self.reader.read(20);
            dataIndex = self.rawdata.find(chr(constants.MESSAGE_TX_SYNC));
            #logging.debug('Total Index:'+str(self.totalIndex)+' Index:'+
            #    str(dataIndex))
            if dataIndex > 0:
                self.totalIndex += dataIndex
                #Make sure you get the whole packet.
                self.reader.seek(self.totalIndex)
                self.rawdata = self.reader.read(20);
                #logging.debug('SEEKED INDEX: '+str(dataIndex)+' Total Index:'
                #+str(self.totalIndex))
            elif dataIndex < 0:
                self.totalIndex += len(self.rawdata)
        try:
            size = self.message.decode(self.rawdata);
        except antException.MessageError as m:
            self.totalIndex += 1;
            if self.totalIndex >= self.fileSize:
                return False;
            else:
                logging.debug('Going into Recursion:'+str(m))
                self.totalIndex +=1
                return self.readline();
        packet = self.rawdata[0: size]
        convertRawDataToHexString(packet)
        timestamp = self.rawdata[size:size+3]
        convertRawDataToHexString(timestamp)

        try:
            hour, min, second = struct.unpack('BBB', \
                timestamp)
        except struct.error:
            logging.warning('ERROR READING FILE.')
            self.totalIndex +=1
            return self.readline();

        
        unixTime = hour*60*60 \
            +min*60 + \
            second
        logging.debug('DECODED MESSAGE TOTAL LENGTH:'+str(size)+\
        ' TOTALINDEX: '+str(self.totalIndex)+'UNIX TIME: '+str(unixTime))
        
        self.totalIndex += size
        #logging.debug('Total Index:'+str(self.totalIndex)+' Index:'+
        #        str(dataIndex))
        return {'timestamp':unixTime,'packet':packet};
    
    def close(self):
        self.reader.close();

    def __iter__(self):
        self.totalIndex = 0
        return self

    def next(self):
        data = self.readline()
        if data == False or data == None:
            raise StopIteration
        return data
