classdef antReader < h5Reader
    %antReader - Reads ANT Cadence/Speed data
    
    properties (Constant)
        tablePreamble = '/AntParsedData'
        lastDirectory = '/sparkfunusb'
    end
    
    methods
        function prD = antReader(filename,runName)
            if nargin > 0 % Support calling with 0 arguments
                prD.filename = filename;
                runName = [runName prD.lastDirectory];
                prD.runName = runName;       
                prD.currentPos = runName;
                disp(prD.currentPos);
            end 
        end %Constructor
        
        function [data,sync] = readRawData(prD,varargin)
           data = prD.readLeafData(...
               [prD.tablePreamble],varargin{:});
         end % readData
         
         function [data sync] = readData(prD,varargin)
           %READDATA Reads the ANT data.
            data = prD.readLeafData(...
               [prD.tablePreamble],varargin{:});
           %data = data(:,5:7);
           sync = prD.readSyncData(varargin{:});
         end % readData
         
         function [speed,cadence,t,sync,sync_t] = readVelocityCadence(prD)
             %readVelocityCadence Get the forward velocity,cadence and
             %sync.
            [data,sync] = prD.readData(false);
            if ~isempty(data)
                speed = data.BikeSpeed;
                cadence = data.BikeCadence;
                if sync ~= false
                    sync_t = sync.Timestamp;
                    sync = sync.Sample;
                else
                    sync_t = [];
                    sync = [];
                end
                t = data.Timestamp;
            else
                speed = [];
                cadence = [];
                t = [];
                sync = [];
                sync_t = [];
            end
         end
    end
end