classdef bdpReader < h5Reader
    %BDPREADER A helper reader to read files created
    %using the https://github.com/moorepants/BicycleDataProcessor
    
    properties (Constant)
        tablePreamble = '/'
        runTable = '/runTable'
    end
    
    methods
        function bdp = bdpReader(filename)
            %Choose the file to access with the run which you would like
            %to read.
            if nargin > 0 % Support calling with 0 arguments
                bdp.filename = filename;
                bdp.runName = '/';
                bdp.currentPos = bdp.runName;
                disp(bdp.currentPos);
            end
        end %Constructor
        
        function data = getRunManeuvers(bdp,varargin)
            data = bdp.readLeafData(...
                [bdp.runTable],false);
            maneaver = bdp.convertColumnToStrings(data.Maneuver);
            ids = double(data.RunID);
            size(ids)
            size(maneaver)
            %ADD check for Corrupt.
            
            %Handlebar - Hit the edge.
            %Knee - knee moved during pertubation.
            %Trailer - True if the trailer bumped edge of treadmill.
            % Array for each pertubation.
            
            
            if ~isempty(varargin) && ~isempty(varargin{1})
                %Filter with respect to value:
                display(['Filtering with: ' varargin{1}]);
                data = zeros(size(maneaver,1),1);
                parfor i = 1:size(maneaver,1)
                    if strcmp(maneaver{i},varargin{1})
                        data(i,1) = ids(i);
                    end
                end
            else
                data = {num2cell(ids),maneaver};
            end
        end % getRunManeuvers
        
        function [theValues,theTableNames] = readRunData(bdp,runID,taskOrRaw)
            if strcmp(class(runID),'str')==false
                runID = num2str(runID,'%05d')
            end
            if taskOrRaw
                runName = '/taskData/'
            else
                runName = '/rawData/'
            end
            runName = [runName, runID, '/']
            display(['Getting Data: ', runName])
           theTables = h5info(bdp.filename,runName);
           theTableNames = cell(length(theTables),1);
           theTableNames{1} = theTables.Datasets(1).Name;
           theColumn = bdp.readLeafData(...
                [runName theTableNames{1}] ,false);
           theValues = zeros(length(theColumn),...
            length(theTables.Datasets));
           theValues(:,1) = theColumn;
           parfor i = 2:length(theTables.Datasets)
               theTableNames{i} = theTables.Datasets(i).Name;
               theColumn = bdp.readLeafData(...
                [runName theTableNames{i}] ,false);
               theValues(:,i) = theColumn;
           end

        end % readRunData
    end
end

