classdef h5Reader < handle
    %   H5READER Class to read the SOFIE hdf5 PyTable file format.
    %
    % A wrapper class to access the SOFIE HDF5 Pytable file format easily
    % and consistently.
    
    properties
        % Used to set the name of the run to read from.
        runName = '';
    end
    
    properties (SetAccess = protected, GetAccess = protected)
        % Used to set the name of the run to read from.
        filename = '';
        currentPos=''
    end
    methods (Static)
        function data = leafData2Array(data)
            % Converts the data to a matrix from astruct.
            %location in the file.
            %
            if isstruct(data)
                data = double(struct2array(data));
            else
                data = data';
            end
        end
        
         function runNames = listRuns(filename)
            %LISTRUNS Lists all the runs in the file.
            %h5dump(h5R.filename,'attr_mode','lite');
            runs = h5info(filename);
            runs = runs.Groups;
            runNames = cell(1,size(runs,1));
            parfor i = 1:size(runs,1)
                runNames{i} = runs(i).Name;
            end
        end
    end
    methods
        function h5R = h5Reader(filename,runName)
            %Choose the file to access with the run which you would like
            %to read.
            if nargin > 0 % Support calling with 0 arguments
                h5R.filename = filename;
                if ~exist(filename,'file')
                    error('sofie:h5ReaderNotExist',...
                        ['The file: ' filename...
                        'does not exist']);
                end
                h5R.runName = runName;
                h5R.currentPos = ['/',runName];
                disp(h5R.currentPos);
            end
        end %Constructor
        
        function h5R = setRunName(h5R,runName)
            h5R.currentPos = runName;
            disp(['Setting Run Name: ',h5R.currentPos]);
            h5R.runName = runName;
        end %set.filename
        
        function runName = getRunName(h5R)
            disp('Getting Run Name');
            runName = h5R.currentPos;
        end %set.filename
        
        function data = readLeafData(h5R,leafName,varargin)
            % Read the data from the the ['/runname/',leafName]
            %location in the file.
            %
            % The 3rd parameter should be set to false
            % if you do not want to convert the data to a matrix from
            % a struct.
            % Returns false if the leaf does not exist.
            disp(h5R.filename)
            disp([h5R.currentPos,'/',leafName])
            error(nargchk(1,3,nargin,'struct'));
            
            try
                data = h5varget(h5R.filename,[h5R.currentPos,'/',leafName]);
            catch exception
                if strcmp(exception.identifier,...
                        'MATLAB:imagesci:hdf5lib:libraryError')==1
                    warning(['The run/leaf: ',h5R.currentPos,'/',...
                        leafName,' is not found in ',h5R.filename]);
                else
                    warning([exception.identifier,' : ', exception.message]);
                end
                data =false;
                return;
            end
            if length(varargin)==1
                if varargin{1} == true
                    data = h5Reader.leafData2Array(data);
                end
            else
                data = h5Reader.leafData2Array(data);
            end
            %data = eval(sprintf('h5R.currentPos.%s',leafName));
            %data
        end % readLeafData 
        
        function column = convertColumnToStrings(h5R,column)
            column = cellstr(column');
        end
        
        function sync = readSyncData(h5R,varargin)
            %Reads the syncdata from the current run.
            sync = h5R.readLeafData(...
               'Sync',varargin{:});
            
        end
    end
    
end

