classdef promoveReader < h5Reader
    %promoveReader - Reads Promove IMU data from the HDF file.
    
    properties (Constant)
        tablePreamble = '/CsvImportData'
    end
    
    methods
        function prD = promoveReader(filename,runName)
            if nargin > 0 % Support calling with 0 arguments
                prD.filename = filename;
                runName = [runName '/csvimport'];
                prD.runName = runName;       
                prD.currentPos = runName;
                disp(prD.currentPos);
            end 
        end %Constructor
        function [quaternions] = readRawNodeData(prD)
            %READRAWNODEDATA Reads raw node data. This is used
            %for a generic quaternions table.
            [quaternions]=prD.readNodeData(-1);
        end
        
        function [quaternions,sync] = readNodeData(prD,nodeId)
            %READNODEDATA Read the quaterernions and timestamp for the
            %node. If note = -1, then it assumes there is only one node
            %and no RTC columns.
            %RETURN
            %quaternion - Nx4 quaternion data is returned.
            %sync - Row vector containing all the points
            % where sync signals were added during recording.
           promoveData = prD.readLeafData(...
               [prD.tablePreamble],false);
           %dataNodeId = promoveData.node0x2Did(promoveData.node0x2Did==nodeId);
           theStructFields = fieldnames(promoveData);
           newFileType =  strcmp('nodeid',theStructFields);
           if (nodeId<0)
               display('READ RAW');
                 quaternions =  [
               promoveData.quat1 ...
               promoveData.quat2 ...
               promoveData.quat3 ...
               promoveData.quat4 ...
               promoveData.timestamp
               ];
               sync = [];  
           elseif any(newFileType)
             quaternions =  [
               promoveData.nodeid ...
               promoveData.quat1 ...
               promoveData.quat2 ...
               promoveData.quat3 ...
               promoveData.quat4 ...
               promoveData.timestamp
               ];
               sync = [
                   promoveData.nodeid promoveData.rtc];
           else
               quaternions =  [
               promoveData.node0x2Did ...
               promoveData.quat1 ...
               promoveData.quat2 ...
               promoveData.quat3 ...
               promoveData.quat4 ...
               promoveData.timestamp
               ];
               sync = [
                   promoveData.node0x2Did promoveData.rtc];
           end
           
         
           
           %Filter only for the current Node.
           if nodeId>0
            quaternions = quaternions(quaternions(:,1)==nodeId,2:6);
            %Get hold of the syncs for the node.
            sync = sync(sync(:,1)==nodeId,2:2);
            sync(isnan(sync)) = 0;
            sync = sync';
           end
           
           %There are some NAN values recorded...
           quaternions(isnan(quaternions)) = 0;
        end % readNode
    end
end


