import logging
from sofiehdfformat.c3dreader.c3dreader import C3DFile
import os
from os import unlink
import struct
import unittest

import sofiehdfformat.c3dreader.c3dreader as C3
from sofiehdfformat.c3dreader.c3dreader import uint32le_to_VAXF_orig
class C3dReaderTest (unittest.TestCase):
    def setUp(self):
        pass
    def tearDown(self):
        pass

    def test_vax(self):
        value = uint32le_to_VAXF_orig(3436035788)
        self.assertEqual(value,-0.10000000149011612)
        value = uint32le_to_VAXF_orig(17136)
        self.assertEqual(value,30)
        value = uint32le_to_VAXF_orig(2147501627)
        self.assertEqual(value,3000)
        
    def test_load_c3d(self):
        c3dFile = C3.open('test-data/test.c3d')
        self.assertEqual(c3dFile.Scale,-0.10000000149011612)
        self.assertEqual(c3dFile.VideoFrameRate,120)
        self.assertEqual(len(c3dFile.ParameterGroups),9)
        self.assertEqual(c3dFile.NvideoFrames,3236)

        Markers = c3dFile.readline();
        MarkerNames = c3dFile.getMarkerNames()
        self.assertEqual(Markers['LBO'].x,501.4815368652344)
        self.assertEqual(Markers[MarkerNames[0]].x,501.4815368652344)
        self.assertEqual(Markers[MarkerNames[5]].x,383.5377502441406)
        self.assertEqual(Markers[MarkerNames[6]].z,1.4693679385278594e-39)
        self.assertEqual(Markers['*8'].z,1.4693679385278594e-39)
        logging.debug(Markers)
        markers = 1;
        read = True;
        first = Markers['LBO'].x
        second = first
        while(read):
            Markers = c3dFile.readline();
            if Markers == False:
                read = False
                continue
            #logging.debug(Markers)
            self.assertEqual(len(Markers),7)
            first = second
            second = Markers
            if second[MarkerNames[0]].x != 1.4693679385278594e-39:
                self.assertNotEqual(first, second)
            markers += 1
        self.assertEqual(c3dFile.NvideoFrames,markers)
        self.assertEqual(second[MarkerNames[0]].x,501.502197265625)
        self.assertEqual(second[MarkerNames[0]].z,960.3297729492188)
        self.assertEqual(second[MarkerNames[5]].x,334.42401123046875)
        self.assertEqual(second[MarkerNames[5]].z,962.0659790039062)
        self.assertEqual(second[MarkerNames[6]].z,1.4693679385278594e-39)
        self.assertEqual(second[MarkerNames[6]].Timestamp,
            markers/c3dFile.VideoFrameRate)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(C3dReaderTest))
    return suite

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    test_suite = suite()
    runner.run (test_suite)